local T, scan_sector = ...

local function get_control_width(generator, str, fontscale)
	local c = iup[generator]{title = str, value = str, value = str, font = Font.H5 * (fontscale or 1), expand = "NO"}
	iup.Map(c)
	local w = tonumber(c.w)
	local h = tonumber(c.h)
	iup.Destroy(c)
	return w, h
end

local function get_color_from_string(color_string, use_string_alpha, color_source, adjust)
	local color_part, alpha
	if (tonumber(color_string)) then
		color_part = color_string
	else
		color_part, alpha = string.match(color_string or "", "^(%d+ %d+ %d+)%s*(%d*)")
	end

	alpha = alpha ~= "" and alpha or nil
	alpha = use_string_alpha and alpha or T.display_alpha
	alpha = math.floor(alpha * (adjust or 1))

	color_string = "255 255 255"

	if     (type(color_part) == "number" and color_source) then
		color_string = color_source[color_part]
	elseif (type(color_part) == "string") then
		color_string = color_part
	end
	return color_string.." "..alpha

end

local function set_selector_highlight(self, index, custom_fgcolor, custom_bgcolor)
	self[index].bgcolor = bgcolor
	self[index].fgcolor = fgcolor
end

local function get_tab_def(tab_index)
	return T.tab_identifiers[tab_index] or {}
end

local function get_tab_filters(tab_index, findex)
	local tab_filters = get_tab_def(tab_index)["filters"]
	return tab_filters and tab_filters[findex] or {}
end

local function get_filter_pages(tab_filters, findex)
	return tab_filters[findex] and tab_filters[findex]["pages"] or {}
end

local function get_filter_colors(tab_index) -- tab_index is already incremented by the caller.
	local findex, fpage = T.tab_filter_positions:GetPositions(tab_index)
	local tab_filters = get_tab_filters(tab_index, findex)
	local tab_pages = get_filter_pages(tab_filters, findex)

tab_filters = T.tab_identifiers[T.tab_index]["filters"]
tab_pages = tab_filters[findex]["pages"]

	local fgcolor = tab_pages and tab_pages[fpage] and tab_pages[fpage]["fgcolor"] or   -- First check if the filter page has a custom color
					tab_filters[findex] and tab_filters[findex]["fgcolor"] -- Next check if the filter has a custom color

	local bgcolor = tab_pages and tab_pages[fpage] and tab_pages[fpage]["bgcolor"] or   -- First check if the filter page has a custom color
					tab_filters[findex] and tab_filters[findex]["bgcolor"] or -- Next check if the filter has a custom color
					T.tab_index == 1 and fpage or           -- Check if it's the first tab and use the filter page to determine color from FactionColor_RGB by index
					1                                             -- default to 0 index for FactionColor_RGB (white)

	return get_color_from_string(fgcolor), get_color_from_string(tonumber(bgcolor) and FactionColor_RGB[bgcolor - 1] or bgcolor)
end




local hud_frame_border_width = "4 "
function hudrightframe(params)
  params.image = IMAGE_DIR .. "hud_border.png"
  params.SEGMENTED = params.SEGMENTED or "0.5 0.5 0.5 0.5"
  params.BORDER = params.BORDER or string.rep(hud_frame_border_width, 4) --"4 4 4 4"
  params.bgcolor = params.bgcolor or get_color_from_string("255 255 255").." &"
  return iup.frame(params)
end


local position = {
	ATOP = {
		active   = ".5 .5 .5 .5", --".5 .5 .5  1",
		inactive = ".5 .5 .5 .5", --" 0  0  1 .5",
		previous = ".5 .5 .5 .5",
	},
	ABOTTOM = {
		active   = ".5 .5 .5 .5",--".5  0 .5 .5",
		inactive = ".5 .5 .5 .5",--" 0 .5  1  1",
		previous = ".5 .5 .5 .5",
	},
}

function T:BuildDisplay()
	local fontsize = Font.H5 * HUD_SCALE * T.fontscale

	local num_width
	local col1_width
	local col2_width = T.name_width
	local col2_height
	local col2_size
	local col3_width
	local filter_height_adjust
	local filter_item_width

	local function display_row_width()
		return num_width + col1_width + col2_width + col3_width
	end

	local function maketab(params)
		local tab
		local tab_active, tab_active_content, label_active
		local tab_inactive, tab_inactive_content, label_inactive
		local tab_label

		local label_params = {
			title = params.title,
			visible = "NO",
			font = fontsize,
			alignment = "ACENTER",
			expand = "YES",
			size = ((params.w or filter_item_width) - hud_frame_border_width).."x"..(fontsize*filter_height_adjust),
			fgcolor = get_color_from_string("255 255 255")
		}

		label_active   = iup.label(label_params)
		tab_active_content = {
			margin = "x"..-hud_frame_border_width,
			iup.pdasubsubsubframebg{
				bgcolor = get_color_from_string("255 255 255").." *",
				iup.vbox{
					iup.fill{size = params.alignment == "ATOP" and 0 or 4},
					label_active,
				},
			},
		}
		if (params.alignment == "ATOP") then
			table.insert(tab_active_content, 1, iup.fill{size = hud_frame_border_width})
		else
			table.insert(tab_active_content, iup.fill{size = hud_frame_border_width})
		end


		tab_active = hudrightframe{
			visible = "YES",
			SEGMENTED = params.SEGMENTED or position[params.alignment or "ATOP"].active,
			drawcenter = params.drawcenter or "NO",
			iup.vbox(tab_active_content),
		}


		label_inactive = iup.label(label_params)
		tab_inactive_content = label_inactive
		tab_inactive = hudrightframe{
			visible = "YES",
			bgcolor = "0 0 0 0",
			SEGMENTED = params.SEGMENTED or position[params.alignment or "ATOP"].inactive,
			drawcenter = params.drawcenter or "NO",
			tab_inactive_content,
		}

		tab = iup.zbox{
			tab_inactive,
			tab_active,
			alignment = "ACENTER",
		}


		function tab:SetVisible(b, c)
			if (b) then
				label_active.visible = "YES"
				label_inactive.visible = "YES"
			else
				label_active.visible = "NO"
				label_inactive.visible = "NO"
			end
		end

		function tab:SetActive(b)
			self.value = self[b and 2 or 1]
		end

		function tab:SetBGColor(color_string, color_source)
			local i = params.alignment == "ATOP" and 2 or 1 -- depending on top or bottom, the fill is in the first or second index
-- 			color_string = get_color_from_string(color_string, nil, color_source).." *"
			color_string = get_color_from_string(color_string, nil, color_source, 0.65).." *"
			tab_active_content[i].bgcolor = color_string
		end

		function tab:SetFGColor(color_string, color_source)
			color_string = get_color_from_string(color_string, nil, color_source).." *"
			label_active.fgcolor = color_string
			label_inactive.fgcolor = color_string
		end

		function tab:SetTitle(t)
			label_active.title = t
			label_inactive.title = t
		end

		return tab
	end

-- -------------------------------------------------------------------
-- -------------------------------------------------------------------
-- -------------------------------------------------------------------

	local function makerow(a, b, c, d, tab_table, init)
		a = a or ""
		b = b or ""
		c = c or ""
		d = d or ""

		local first_run = a == 99
		local row, row_content, shield, defense
		local cols = {}

		if (not num_width) then
			num_width  = get_control_width("label", a, HUD_SCALE * T.fontscale)
			col1_width = get_control_width("label", b, HUD_SCALE * T.fontscale)
			local c2w, c2h = get_control_width("text", c, HUD_SCALE * T.fontscale)
			col2_width = col2_width or c2w
			T.defaults.name_width.val = col2_width
			col3_width = get_control_width("label", d, HUD_SCALE * T.fontscale)
		end

		if (tab_table) then
			local row_width = col1_width + col2_width + col3_width
			local sum_tab_width_pct = 0 -- current sum of column percents.  Should not exceed 1.
			local font = tab_table.font and tab_table.font * HUD_SCALE * T.fontscale or fontsize

			for i = 1, 3 do
				local col_info = tab_table.columns and tab_table.columns[i] or {}
				-- verify and adjust column percent as necessary
				local w = col_info.width or 0
				w = (w > 1 and 1) or (w < 0 and 0) or w -- adjust the width to be within min-max if necessary
				w = (sum_tab_width_pct + w) > 1 and (1 - sum_tab_width_pct) or w -- adjust the width down if over max
				w = i == 3 and (sum_tab_width_pct + w) < 1 and (1 - sum_tab_width_pct) or w -- fill up the remaining space in the last column if the entire width is unused
				sum_tab_width_pct = sum_tab_width_pct + w

				if ( w == 0) then
					cols[i] = iup.fill{size = 0}
				else
					cols[i] = iup.text{value = col_info.title or "", size = row_width * w, font = font, active = "NO", border = "NO", fgcolor = get_color_from_string("255 255 255").." *", bgcolor = "0 0 0 255"}
				end
			end

		else
			cols[1] = iup.label{title = b, wordwrap = "NO", font = fontsize, expand = "NO", size = col1_width, fgcolor = get_color_from_string("255 255 255").." *"}
			cols[2] = iup.text {value = c, wordwrap = "NO", font = fontsize, expand = "NO", size = col2_width, fgcolor = get_color_from_string("255 255 255").." *", active = "NO", border = "NO", bgcolor = "0 0 0 255"}
			cols[3] = iup.label{title = d, wordwrap = "NO", font = fontsize, expand = "NO", size = col3_width, fgcolor = get_color_from_string("255 255 255").." *", alignment = "ARIGHT"}

			defense = iup.stationprogressbar{
				visible = "NO", active = "NO", title = "", expand = "NO",--"HORIZONTAL",
				size = col2_width.."x"..math.ceil(fontsize/10),
				minvalue = 0, maxvalue = 100, uppercolor = get_color_from_string("88 88 88").." *", lowercolor = get_color_from_string("255 65 65").." *",
				value = 50, uv = "0 0.25 1 0.75"
			}

			shield = iup.stationprogressbar{
				visible = "NO", active = "NO", title = "", expand = "NO", --"HORIZONTAL",
				size = col2_width.."x"..math.ceil(fontsize/4),
				minvalue = 0, maxvalue = 100, uppercolor = get_color_from_string("88 88 88").." *", lowercolor = get_color_from_string("65 65 255").." *",
				value = 50, uv = "0 0.25 1 0.75"
			}

			if (first_run) then cols[2].value = cols[2].value:gsub("X", "") end
		end

		local num  = iup.label{title = a, wordwrap = "NO", font = fontsize, expand = "NO", size = num_width, fgcolor = get_color_from_string().."*", alignment = "ACENTER"}
		if (first_run) then num.fgcolor = "0 0 0 0 *" end

		local num_highlight = iup.pdasubsubsubframefull2{
			iup.vbox{
				iup.fill{size = 2},
				iup.hbox{
					num,
					margin = "-4x-4"
				},
			},
			bgcolor = get_color_from_string("0 0 0 0", true).." *"
		}

		row_content = iup.hbox{
			num_highlight,
			cols[1],
			iup.zbox{
				iup.vbox{
					iup.fill{},
					shield,
					defense,
					--gap = math.ceil(fontsize*1/4/2),
					margin = "x-.5", --..math.ceil(fontsize*1/4/2),
					expand = "NO",
				},
				cols[2],
				all = "YES",
				alignment = "SW",
			},
			cols[3],
			gap = T.display_column_gap,
			margin = "-2x-2"
		}

		row = iup.pdasubsubsubframefull2{bgcolor = "0 0 0 0 *", visible = "NO", row_content}

		function row:GetSize()
			return cols[2].w, cols[2].h, cols[2].x, cols[2].y
		end

		function row:Resize(x, y, r) -- receives X and Y positions fromm the scrollbar.  Need to convert those into width and height
			local w = x >= T.name_width_min and x or T.name_width_min
			local h
			cols[2].size = (w or cols[2].w).."x"..(h or cols[2].h)
			defense.size = (w or defense.w).."x"..(h or defense.h)
			shield.size = (w or shield.w).."x"..(h or shield.h)
			if (r) then iup.Refresh(cols[2]) end
		end

		function row:SetNumberFGColor(color_string)
			num.fgcolor = get_color_from_string(color_string).." *"
		end
		function row:SetNumberBGColor(color_string)
			num_highlight.bgcolor = get_color_from_string(color_string, true).." *"
		end

		function row:SetVisible(b)
			self.visible = b and "YES" or "NO"
		end
		function row:SetHighlight(color_string)
			color_string = get_color_from_string(color_string, true).." *" -- need T.display_alpha 0 if no color provided;  might be a better way to handle this
			self.bgcolor = color_string
		end
		function row:SetDefense(v, color_string)
			defense.visible = v > -1 and "YES" or "NO"
			defense.value = v
			defense.lowercolor = get_color_from_string(color_string, nil).." *"
		end
		function row:SetShields(v)
			shield.visible = v > -1 and "YES" or "NO"
			shield.value = v
		end

		local column_actions = {
			function(v, c)
				cols[1].title = v
				cols[1].value = v
				cols[1].fgcolor = get_color_from_string(c, true).." *"
				if (iup.GetType(cols[1]) == "text") then cols[1].caret = 1 end
			end,
			function(v, c)
				cols[2].value = v
				cols[2].fgcolor = get_color_from_string(c, true).." *"
				if (iup.GetType(cols[2]) == "text") then cols[2].caret = 1 end
			end,
			function(v, c)
				cols[3].title = v
				cols[3].value = v
				cols[3].fgcolor = get_color_from_string(c, true).." *"
				if (iup.GetType(cols[3]) == "text") then cols[3].caret = 1 end
			end
		}

		function row:SetColumn(i, v, c)
			column_actions[i](v, c)
		end


		return row
	end



-- -------------------------------------------------------------------
-- ---                                                             ---
-- --- Display Code                                                ---
-- ---                                                             ---
-- -------------------------------------------------------------------


	local dlg
	local container, display_container, tab_container, filter_container
	local tab_box, tabs, filter_box, filters,  row_box, rows

-- -------------------------------------------------------------------
-- Build display rows -----------------------------------------------
-- -------------------------------------------------------------------
	if (T.tab_index > 0) then
		-- populate the column widths if the tab is not "Targets"
		makerow(99, "Health", "Name"..string.rep("X", 13), "Distance")
	end
	local rows = {
		makerow(99, "Health", "Name"..string.rep("X", 13), "Distance", T.tab_index > 1 and T.tab_identifiers[T.tab_index]),
		margin = "10x5",
		gap = T.display_row_gap,
	}
	rows[1]:SetVisible(true)

	for i = 1, T.list_count do
		table.insert(rows, makerow(i, "", "", "", T.tab_index > 1 and T.tab_identifiers[T.tab_index]))
	end

	function rows:GetSize()
		return self[1]:GetSize()
	end

	function rows:Resize(x, y) -- receives X and Y positions fromm the scrollbar.  Need to convert those into width and height
		for i, r in ipairs(rows) do
			r:Resize(x, y)
		end
		iup.Refresh(iup.GetParent(rows[1]))
	end

	T.UI.display = rows

-- -------------------------------------------------------------------
-- Build tabs --------------------------------------------------------
-- -------------------------------------------------------------------

	local tab_indicator_width = get_control_width("label", "<")
	local tab_item_width = (display_row_width() - tab_indicator_width * 2) / T.tab_window_size
	filter_item_width = (display_row_width()) / T.tab_window_size
	filter_height_adjust = fontsize > 13 and 4/5 or 1

	local tabs = iup.hbox{
		maketab{title = "<",   alignment = "ATOP", w = tab_indicator_width},--, drawcenter = "YES"},
		maketab{title = "aaa", alignment = "ATOP", w = tab_item_width},
		maketab{title = "bbb", alignment = "ATOP", w = tab_item_width},
		maketab{title = "ccc", alignment = "ATOP", w = tab_item_width},
		maketab{title = "ddd", alignment = "ATOP", w = tab_item_width},
		maketab{title = ">",   alignment = "ATOP", w = tab_indicator_width},--, drawcenter = "YES"},
	}
	tab_box = iup.pdarootframebg{tabs, bgcolor = "0 0 0 0 *"}
	-- Build the tab display
	table.sort(T.tab_identifiers, function(a, b)
		if (a == T.Tab) then return true end
		if (b == T.Tab) then return false end
		return a.title:lower() < b.title:lower()
	end)

	local num_tabs = #T.tab_identifiers

	-- Tabs to the left indicator
	if (T.tab_window_offset ~= 0) then
		tabs[1]:SetVisible(true)
	else
		tabs[1]:SetVisible(false)
	end

	-- Tabs to the right indicator
	if     (T.tab_window_offset == num_tabs - T.tab_window_size) then
		tabs[6]:SetVisible(false)
	elseif (num_tabs > T.tab_window_size) then
		tabs[6]:SetVisible(true)
	end

	for i = 1, T.tab_window_size do
		local tab_tbl = T.tab_identifiers[i + T.tab_window_offset]
		if (tab_tbl) then
			local tab = tabs[i + 1]
			tab:SetTitle(tab_tbl["title"])
			tab:SetFGColor(tab_tbl["fgcolor"], T.tab_identifiers[T.tab_index]["colorsource"])
			tab:SetBGColor(tab_tbl["bgcolor"], T.tab_identifiers[T.tab_index]["colorsource"])
			tab:SetVisible(true)
		else
			break
		end
	end

	-- Activate the current tab
	tabs[T.tab_index + 1 - T.tab_window_offset]:SetActive(true) -- +1 for the left indicator

	-- Don't show any tabs if only the "Targets" tab is registered
	if (num_tabs == 1) then
		tabs[2]:SetVisible(false)
		tabs[2]:SetActive(false)
	end

	T.UI.tabs = tabs

-- -------------------------------------------------------------------
-- --- Build filters -------------------------------------------------
-- -------------------------------------------------------------------
	local filters = iup.hbox{
		maketab{title = "aaa", alignment = "ABOTTOM"},
		maketab{title = "bbb", alignment = "ABOTTOM"},
		maketab{title = "ccc", alignment = "ABOTTOM"},
		maketab{title = "ddd", alignment = "ABOTTOM"},
	}
	-- Build the filters for the current tab

	local cur_filter_pages = T.tab_filter_positions[T.tab_identifiers[T.tab_index]]
	cur_filter_pages = cur_filter_pages and cur_filter_pages["pages"]
	local filter_list = T.tab_identifiers[T.tab_index]["filters"] or {}
	for i, filter in ipairs(filter_list) do
		if (i > 4) then break end
		local pages = filter["pages"]
		local fpage = type(cur_filter_pages) == "table" and cur_filter_pages[i] or cur_filter_pages
		local link = filter_list["link_filter_positions"]

		local title
		local fgcolor
		local bgcolor

		if (pages) then
			title   = link and filter["title"] or pages[fpage]["title"]
			fgcolor = link and filter_list[1]["pages"][fpage]["fgcolor"] or pages[fpage]["fgcolor"]
			bgcolor = link and filter_list[1]["pages"][fpage]["bgcolor"] or pages[fpage]["bgcolor"]
		else
			title = filter["title"]
		end

		if (not fgcolor) then fgcolor = filter["fgcolor"] end
		if (not bgcolor) then bgcolor = filter["bgcolor"] end

		filters[i]:SetTitle(title)
		filters[i]:SetFGColor(fgcolor, T.tab_identifiers[T.tab_index]["colorsource"])
		filters[i]:SetBGColor(bgcolor, T.tab_identifiers[T.tab_index]["colorsource"])
		filters[i]:SetVisible(true)
	end

	local findex, fpage = T.tab_filter_positions:GetPositions(T.tab_index)
	if (findex) then filters[findex]:SetActive(true) end


	filter_box = iup.pdarootframebg{
		iup.zbox{
			iup.pdasubsubsubframefull2{
				iup.vbox{
					iup.label{title = "", expand = "YES", visible = "NO", bgcolor = "0 0 0 0 *"},
					margin = "-6x-6",
				},
				bgcolor = "255 0 0 "..(T.tab_index == 1 and T.invert_faction_filter and 128 or 0).." *"
			},
			filters,
			all = "YES",
		},
		bgcolor = "0 0 0 0 *",
	}

	T.UI.filters = filters

-- -------------------------------------------------------------------
-- --- Build display container ---------------------------------------
-- -------------------------------------------------------------------

	display_container = iup.pdarootframebg{ -- vboxes have fullscreen size.  A frame or other drawn control is necessary for sizing various things
		iup.vbox{
			tab_box,
			hudrightframe{
				iup.vbox(rows),
				SEGMENTED = ".5 .5 .5 .5",
 				drawcenter = T.display_drawcenter and "YES" or "NO",
			},
			filter_box,
		},

		cx = 0,
		cy = 0,
		expand = "NO",
		bgcolor = "0 0 0 0 *", -- hide the containing frame
	}
T.UI.display_container = display_container

-- -------------------------------------------------------------------
-- ---                                                             ---
-- --- Support Code                                                ---
-- ---                                                             ---
-- -------------------------------------------------------------------


	local function update_target_list(i, list)
		local health, shield, name = "", "", ""
		local charID = list[i]["charID"]
		local nodeID = GetPlayerNodeID(charID)

		name = list[i]["name"]
		list[i]["nodeID"] = nodeID
		list[i]["objectID"] = GetPrimaryShipIDOfPlayer(charID)
		health, shield = GetPlayerHealth(charID)
		health = math.ceil(health or 0)
		local guild = GetGuildTag(charID)
		if (guild and guild ~= "") then name = "["..tostring(guild).."] "..name end
		local row = rows[i + T.list_offset]

		local defender = T.defenselist[charID]
		local aggressor = T.aggressorlist[charID]
		if (defender or aggressor) then
			local color_string = defender and "65 255 65" or aggressor and "255 65 65"
			local time = os.time() - (defender or aggressor)
			if (time <= T.defense_timeout) then
				time = 100 - 100 / T.defense_timeout * time
			else
				T.defenselist[charID] = nil
				T.aggressorlist[charID] = nil
				time = -1
			end
			row:SetDefense(math.floor(time), color_string)
		else
			row:SetDefense(-1)
		end

		row:SetColumn(1, health == -1 and "" or health.."%", calc_health_color(health * 0.01))
		row:SetColumn(2, name, FactionColor_RGB[list[i]["factionID"]])
		row:SetColumn(3, list[i].distance == 9999 and "" or list[i]["distance"])
		row:SetShields(shield and list[i].capship and math.ceil(shield) or -1)

		if (T.pinlist[charID]) then
			row:SetNumberBGColor("255 0 0")
		else
			row:SetNumberBGColor("0 0 0 0")
		end

		if (nodeID == T.current_target_nodeid) then
			row:SetHighlight(T.current_target_color)
			T.current_target_listindex = i
		else
			row:SetHighlight("0 0 0 0")
		end
	end

	-- Update the display of non-Target Tabs
	local function update_alt_tab(i, list)
		local row = rows[i + T.list_offset]
		local list_item = list[i] or {}
		-- iterate over the three columns for the list display
		for j = 1, 3 do
			row:SetColumn(j, list_item[j] or "")
		end

		-- set the highlight if necessary
		if (list_item.highlight) then
			row:SetHighlight(T.current_target_color)
			T.current_target_listindex = i
		else
			row:SetHighlight("0 0 0 0")
		end
	end

	-- Update the targets list
	function display_container.UpdateDisplay(list, alt)
		T.tab_target_list = list
		T.current_target_listindex = nil
		local display_func = alt and update_alt_tab or update_target_list

		for i = 1, T.list_count do

			if (list[i]) then

				display_func(i, list)
				rows[i + T.list_offset]:SetVisible(true)
			else
				rows[i + T.list_offset]:SetVisible(false)
			end
		end
	end

	function display_container:PositionDisplay(x, y)
		x = tonumber(x) or (HUD.selfinfoframe.x - display_row_width() - 50)
		y = tonumber(y) or (HUD.groupinfoframe.y)
		self.cx = x
		self.cy = y
		HUD.cboxlayer.REFRESH = "YES"
		return x, y -- return x, y in case it was reset to default
	end


	iup.Append(iup.GetNextChild(HUD.cboxlayer), display_container)
	HUD.cboxlayer:map()
	display_container:PositionDisplay(T.x, T.y)
 end


function T:DestroyDisplay()
	T.refresh_timer:Kill()
	iup.Detach(T.UI.display_container)
	iup.Destroy(T.UI.display_container)
	T.UI.display_container = nil
	T.UI.display = nil
	T.UI.filters = nil
	T.UI.tabs = nil
	self:DestroyRegions()
end

function T:RebuildDisplay()
	-- Reset the filter positions between tab shifts
	self:DestroyDisplay()
	self:BuildDisplay()
	if (self.tab_index == 1) then
	end
	self:CreateRegions()
	if (T.tab_index == 1) then
		T.UI.display[1]:SetDefense(-1)
		T.UI.display[1]:SetShields(-1)
		scan_sector()   -- can this be moved?
	end
end
